/**
 * Copyright (c) 2013, MindFusion LLC - Bulgaria.
 */

package samples.effects;

import java.awt.*;
import java.awt.event.*;
import java.awt.event.ItemEvent;
import java.beans.*;
import java.text.*;
import java.util.*;

import javax.imageio.*;
import javax.swing.*;
import javax.swing.SpringLayout;
import javax.swing.event.*;

import com.mindfusion.diagramming.*;
import com.mindfusion.diagramming.Shape;


public class MainFrame extends JFrame
	implements ChangeListener, ListSelectionListener, ItemListener, PropertyChangeListener, ActionListener 
{
	public MainFrame()
	{
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setLocationByPlatform(true);
		setSize(1024, 768);
		setTitle("JDiagram Sample: Effects");
		try
		{
			ArrayList<Image> images = new ArrayList<Image>();
			images.add(ImageIO.read(App.class.getResourceAsStream("App.png")));
			images.add(ImageIO.read(App.class.getResourceAsStream("App32.png")));
			setIconImages(images);
		}
		catch (Exception ex)
		{
		}

		// Create effects
		glassEffect = new GlassEffect();
		aeroEffect = new AeroEffect();

		initializeComponent();

		// Defaults
		diagram.getDiagramStyle().setShadowBrush(new SolidBrush(new Color(0, 0, 0, 100)));
		diagram.setShadowsStyle(ShadowsStyle.None);

		// Presets
		defaultPreset = new Preset();
		defaultPreset.Name = "(Default)";
		defaultPreset.UseGlassEffect = false;
		defaultPreset.GlassEffectType = GlassEffectType.Type1;
		defaultPreset.UsePenAsGlow = false;
		defaultPreset.GlowColor = Colors.White;
		defaultPreset.ReflectionColor = Colors.White;
		defaultPreset.UseAeroEffect = false;
		defaultPreset.Opacity = 40;
		defaultPreset.InnerOutlineColor = Colors.White;
		defaultPreset.ShadeColor = Colors.Black;
		defaultPreset.DiagramBackground = Colors.White;
		defaultPreset.ShapeBorder = Colors.DarkSlateGray;
		defaultPreset.ShapeBackground1 = Colors.LightSteelBlue;
		defaultPreset.UseShapeBackground2 = false;
		defaultPreset.ShapeBackground2 = Colors.White;
		applyPreset(defaultPreset);

		Vector<Preset> presets = new Vector<Preset>();
		Preset preset = new Preset();
		preset.Name = "Glass (Default Colors)";
		preset.UseGlassEffect = true;
		preset.GlassEffectType = GlassEffectType.Type1;
		preset.UsePenAsGlow = false;
		preset.GlowColor = Colors.White;
		preset.ReflectionColor = Colors.White;
		preset.UseAeroEffect = false;
		preset.Opacity = 40;
		preset.InnerOutlineColor = Colors.White;
		preset.ShadeColor = Colors.Black;
		preset.DiagramBackground = Colors.White;
		preset.ShapeBorder = Colors.DarkSlateGray;
		preset.ShapeBackground1 = Colors.LightSteelBlue;
		preset.UseShapeBackground2 = false;
		preset.ShapeBackground2 = Colors.White;
		presets.add(preset);

		preset = new Preset();
		preset.Name = "Glass & Aero (Orange Background)";
		preset.UseGlassEffect = true;
		preset.GlassEffectType = GlassEffectType.Type2;
		preset.UsePenAsGlow = false;
		preset.GlowColor = Colors.White;
		preset.ReflectionColor = Colors.White;
		preset.UseAeroEffect = true;
		preset.Opacity = 20;
		preset.InnerOutlineColor = Colors.White;
		preset.ShadeColor = Colors.Black;
		preset.DiagramBackground = Colors.Orange;
		preset.ShapeBorder = Colors.DarkSlateGray;
		preset.ShapeBackground1 = Colors.LightSteelBlue;
		preset.UseShapeBackground2 = false;
		preset.ShapeBackground2 = Colors.White;
		presets.add(preset);

		preset = new Preset();
		preset.Name = "Metalic";
		preset.UseGlassEffect = true;
		preset.GlassEffectType = GlassEffectType.Type2;
		preset.UsePenAsGlow = true;
		preset.GlowColor = Colors.White;
		preset.ReflectionColor = Colors.White;
		preset.UseAeroEffect = true;
		preset.Opacity = 50;
		preset.InnerOutlineColor = Colors.White;
		preset.ShadeColor = Colors.Black;
		preset.DiagramBackground = new Color(0xC0, 0xC0, 0xC0);
		preset.ShapeBorder = Colors.DarkSlateGray;
		preset.ShapeBackground1 = Colors.LightSteelBlue;
		preset.UseShapeBackground2 = false;
		preset.ShapeBackground2 = Colors.White;
		presets.add(preset);

		preset = new Preset();
		preset.Name = "Neon Glow";
		preset.UseGlassEffect = true;
		preset.GlassEffectType = GlassEffectType.Type2;
		preset.UsePenAsGlow = true;
		preset.GlowColor = Colors.White;
		preset.ReflectionColor = Colors.White;
		preset.UseAeroEffect = true;
		preset.Opacity = 0;
		preset.InnerOutlineColor = Colors.Black;
		preset.ShadeColor = new Color(0x80, 0xFF, 0xFF);
		preset.DiagramBackground = Colors.Black;
		preset.ShapeBorder = new Color(0x80, 0xFF, 0xFF);
		preset.ShapeBackground1 = Colors.Black;
		preset.UseShapeBackground2 = false;
		preset.ShapeBackground2 = Colors.White;
		presets.add(preset);

		preset = new Preset();
		preset.Name = "Gray glass";
		preset.UseGlassEffect = true;
		preset.GlassEffectType = GlassEffectType.Type3;
		preset.UsePenAsGlow = false;
		preset.GlowColor = Colors.White;
		preset.ReflectionColor = Colors.White;
		preset.UseAeroEffect = true;
		preset.Opacity = 40;
		preset.InnerOutlineColor = Colors.White;
		preset.ShadeColor = Colors.Black;
		preset.DiagramBackground = Colors.Gray;
		preset.ShapeBorder = Colors.Black;
		preset.ShapeBackground1 = Colors.LightCyan;
		preset.UseShapeBackground2 = true;
		preset.ShapeBackground2 = new Color(0xA8, 0xBF, 0xBF);
		presets.add(preset);

		preset = new Preset();
		preset.Name = "Brown";
		preset.UseGlassEffect = true;
		preset.GlassEffectType = GlassEffectType.Type1;
		preset.UsePenAsGlow = false;
		preset.GlowColor = Colors.White;
		preset.ReflectionColor = Colors.White;
		preset.UseAeroEffect = false;
		preset.Opacity = 40;
		preset.InnerOutlineColor = Colors.White;
		preset.ShadeColor = Colors.Black;
		preset.DiagramBackground = Colors.PaleGoldenrod;
		preset.ShapeBorder = Colors.Black;
		preset.ShapeBackground1 = new Color(0x90, 0x30, 0x20, 0x80);
		preset.UseShapeBackground2 = false;
		preset.ShapeBackground2 = Colors.White;
		presets.add(preset);

		preset = new Preset();
		preset.Name = "Bright Red";
		preset.UseGlassEffect = true;
		preset.GlassEffectType = GlassEffectType.Type4;
		preset.UsePenAsGlow = false;
		preset.GlowColor = Colors.Black;
		preset.ReflectionColor = Colors.White;
		preset.UseAeroEffect = false;
		preset.Opacity = 40;
		preset.InnerOutlineColor = Colors.White;
		preset.ShadeColor = Colors.Black;
		preset.DiagramBackground = Colors.White;
		preset.ShapeBorder = new Color(0x8E, 0x8F, 0x8F);
		preset.ShapeBackground1 = Colors.Red;
		preset.UseShapeBackground2 = false;
		preset.ShapeBackground2 = Colors.White;
		presets.add(preset);

		presetsList.setListData(presets);

		// Preview shapes
		int i = 0;
		final int size = 20;
		final int perLine = 7;
		for (Shape shape: Shape.getShapes())
		{
			if (shape.getIsArrowhead())
				continue;

			// BPMN shapes don't add variety
			if (shape.getId().startsWith("Bpmn"))
				continue;

			ShapeNode node = diagram.getFactory().createShapeNode((i % perLine) * (size + 5) + 4,
				(i / perLine) * (size + 5) + 4, size, size);
			node.setShape(shape);
			i++;
		}
		diagram.resizeToFitItems(4, false);
	}

	private void initializeComponent()
	{
		setLayout(new BorderLayout());

		panel = new JPanel(new VerticalFlowLayout(VerticalFlowLayout.LEFT, VerticalFlowLayout.TOP, 5, 0));
		presetsList = new JList<Preset>();
		panel2 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		checkBox1 = new JCheckBox("Glass Effect");
		glassEffectPanel = new JPanel(new SpringLayout());
		label1 = new JLabel("Type:");
		comboBox1 = new JComboBox<String>();
		checkBox2 = new JCheckBox("Use Pen as Glow");
		label2 = new JLabel("Glow color:");
		colorPicker1 = new ColorPicker();
		label3 = new JLabel("Reflection color:");
		colorPicker2 = new ColorPicker();
		panel3 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		checkBox3 = new JCheckBox("Aero Effect");
		aeroEffectPanel = new JPanel(new SpringLayout());
		label7 = new JLabel("Opacity (%):");
		numeric1 = new JFormattedTextField(NumberFormat.getInstance());
		label6 = new JLabel("Inner outline color:");
		colorPicker4 = new ColorPicker();
		label5 = new JLabel("Shade color:");
		colorPicker3 = new ColorPicker();
		bottomPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		panel6 = new JPanel(new SpringLayout());
		label4 = new JLabel("Diagram background:");
		colorPicker5 = new ColorPicker();
		label9 = new JLabel("Shapes border:");
		colorPicker7 = new ColorPicker();
		label8 = new JLabel("Shapes background:");
		panel7 = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 2));
		colorPicker6 = new ColorPicker();
		checkBox6 = new JCheckBox();
		colorPicker8 = new ColorPicker();
		diagramView = new DiagramView();
		diagramPane = new JScrollPane(diagramView);
		// panel
		panel.setPreferredSize(new Dimension(250, 250));
		panel.add(Box.createVerticalStrut(10));
		panel.add(new JLabel("Presets:"));
		panel.add(Box.createVerticalStrut(5));
		panel.add(presetsList, Integer.valueOf(VerticalFlowLayout.STRETCH));
		panel.add(Box.createVerticalStrut(5));
		panel.add(panel2, Integer.valueOf(VerticalFlowLayout.STRETCH));
		panel.add(glassEffectPanel, Integer.valueOf(VerticalFlowLayout.STRETCH));
		panel.add(Box.createVerticalStrut(5));
		panel.add(panel3, Integer.valueOf(VerticalFlowLayout.STRETCH));
		panel.add(aeroEffectPanel, Integer.valueOf(VerticalFlowLayout.STRETCH));
		panel.add(Box.createVerticalStrut(5));
		// presetsList
		presetsList.setPreferredSize(new Dimension(230, 203));
		presetsList.setBorder(BorderFactory.createLineBorder(Colors.Gray));
		presetsList.addListSelectionListener(this);
		// checkBox1
		checkBox1.setOpaque(false);
		checkBox1.setAlignmentX(0);
		checkBox1.addItemListener(this);
		// panel2
		panel2.add(checkBox1);
		panel2.setBackground(new Color(0xB9, 0xD1, 0xEA));
		// glassEffectPanel
		glassEffectPanel.setBackground(new Color(0xD7, 0xE4, 0xF2));
		glassEffectPanel.setPreferredSize(new Dimension(231, 122));
		glassEffectPanel.add(label1);
		glassEffectPanel.add(comboBox1);
		glassEffectPanel.add(new JLabel());
		glassEffectPanel.add(checkBox2);
		glassEffectPanel.add(label2);
		glassEffectPanel.add(colorPicker1);
		glassEffectPanel.add(label3);
		glassEffectPanel.add(colorPicker2);
		makeCompactGrid(glassEffectPanel, 4, 2, 10, 10, 5, 5);
		// label1
		label1.setOpaque(false);
		label1.setLabelFor(comboBox1);
		// comboBox1
		comboBox1.addItem("Type 1");
		comboBox1.addItem("Type 2");
		comboBox1.addItem("Type 3");
		comboBox1.addItem("Type 4");
		comboBox1.addItemListener(this);
		// checkBox2
		checkBox2.setOpaque(false);
		checkBox2.addItemListener(this);
		// label2
		label2.setOpaque(false);
		// colorPicker1
		colorPicker1.addPropertyChangeListener(this);
		// label2
		label2.setLabelFor(colorPicker1);
		// label3
		label3.setOpaque(false);
		label3.setLabelFor(colorPicker2);
		// colorPicker2
		colorPicker2.addPropertyChangeListener(this);
		// panel3
		panel3.setBackground(new Color(0xB9, 0xD1, 0xEA));
		panel3.add(checkBox3);
		// checkBox3
		checkBox3.setOpaque(false);
		checkBox3.setAlignmentX(0);
		checkBox3.addItemListener(this);
		// aeroEffectPanel
		aeroEffectPanel.setBackground(new Color(0xD7, 0xE4, 0xF2));
		aeroEffectPanel.setPreferredSize(new Dimension(231, 93));
		aeroEffectPanel.add(label7);
		aeroEffectPanel.add(numeric1);
		aeroEffectPanel.add(label2);
		aeroEffectPanel.add(colorPicker4);
		aeroEffectPanel.add(label5);
		aeroEffectPanel.add(colorPicker3);
		makeCompactGrid(aeroEffectPanel, 3, 2, 10, 10, 5, 5);
		// label7
		label7.setOpaque(false);
		label7.setLabelFor(numeric1);
		// numeric1
		numeric1.addPropertyChangeListener(this);
		// label6
		label6.setOpaque(false);
		label6.setLabelFor(colorPicker4);
		// colorPicker4
		colorPicker4.addPropertyChangeListener(this);
		// label5
		label5.setOpaque(false);
		label5.setLabelFor(colorPicker3);
		// colorPicker3
		colorPicker3.setColor(Color.BLACK);
		colorPicker3.addPropertyChangeListener(this);
		// bottomPanel
		bottomPanel.add(panel6);
		// panel6
		panel6.setPreferredSize(new Dimension(320, 91));
		panel6.add(label4);
		panel6.add(colorPicker5);
		panel6.add(label9);
		panel6.add(colorPicker7);
		panel6.add(label8);
		panel6.add(panel7);
		makeCompactGrid(panel6, 3, 2, 0, 0, 5, 5);
		// label4
		label4.setOpaque(false);
		label4.setLabelFor(colorPicker5);
		// colorPicker5
		colorPicker5.addPropertyChangeListener(this);
		// label9
		label9.setOpaque(false);
		label9.setLabelFor(colorPicker7);
		// colorPicker7
		colorPicker7.addPropertyChangeListener(this);
		// label8
		label8.setOpaque(false);
		label8.setLabelFor(colorPicker6);
		// colorPicker6
		colorPicker6.addPropertyChangeListener(this);
		// panel7
		panel7.add(colorPicker6);
		panel7.add(checkBox6);
		panel7.add(colorPicker8);
		// checkBox6
		checkBox6.setOpaque(false);
		checkBox6.addItemListener(this);
		// colorPicker8
		colorPicker8.addPropertyChangeListener(this);
		// diagram
		diagram = diagramView.getDiagram();

		// Structure
		Container c = getContentPane();
		c.add(panel, BorderLayout.EAST);
		c.add(bottomPanel, BorderLayout.SOUTH);
		c.add(diagramPane, BorderLayout.CENTER);
	}

	/**
	 * {@link ChangeListener#stateChanged} implementation.
	 */
	public void stateChanged(ChangeEvent e)
	{
	}

	/**
	 * {@link ListSelectionListener#valueChanged} implementation.
	 */
	public void valueChanged(ListSelectionEvent e)
	{
		if (e.getSource() == presetsList)
		{
			// Apply the selected preset
			applyPreset(presetsList.getSelectedValue());
		}
	}

	/**
	 * {@link ItemListener#itemStateChanged} implementation.
	 */
	public void itemStateChanged(ItemEvent e)
	{
		if (e.getSource() == comboBox1)
		{
			glassEffect.setType(comboBox1.getSelectedIndex());
		}
		else if (e.getSource() == checkBox1)
		{
			if (checkBox1.isSelected())
				diagram.getNodeEffects().add(glassEffect);
			else
				diagram.getNodeEffects().remove(glassEffect);
		}
		else if (e.getSource() == checkBox2)
		{
			glassEffect.setUsePenAsGlow(checkBox2.isSelected());
		}
		else if (e.getSource() == checkBox3)
		{
			if (checkBox3.isSelected())
				diagram.getNodeEffects().add(aeroEffect);
			else
				diagram.getNodeEffects().remove(aeroEffect);
		}
		else if (e.getSource() == checkBox6)
		{
			updateShapeBrush();
			colorPicker8.setEnabled(checkBox6.isSelected());
		}
	}

	/**
	 * {@link PropertyChangeListener#propertyChange} implementation.
	 */
	public void propertyChange(PropertyChangeEvent evt)
	{
		if ("color".equals(evt.getPropertyName()))
		{
			if (evt.getSource() == colorPicker1)
				glassEffect.setGlowColor(colorPicker1.getColor());
			else if (evt.getSource() == colorPicker2)
				glassEffect.setReflectionColor(colorPicker2.getColor());
			else if (evt.getSource() == colorPicker3)
				aeroEffect.setShadeColor(colorPicker3.getColor());
			else if (evt.getSource() == colorPicker4)
				aeroEffect.setInnerOutlineColor(colorPicker4.getColor());
			else if (evt.getSource() == colorPicker5)
				diagram.setBackBrush(new SolidBrush(colorPicker5.getColor()));
			else if (evt.getSource() == colorPicker7)
			{
				SolidBrush stroke = new SolidBrush(colorPicker7.getColor());
				diagram.getShapeNodeStyle().setStroke(stroke);
			}
			else if (evt.getSource() == colorPicker6 || evt.getSource() == colorPicker8)
				updateShapeBrush();
		}
		if ("value".equals(evt.getPropertyName()))
		{
			if (evt.getSource() == numeric1)
				aeroEffect.setOpacity(Float.parseFloat(String.valueOf(numeric1.getValue())) / 100);
		}
	}

	/**
	 * {@link ActionListener#actionPerformed} implementation.
	 */
	public void actionPerformed(ActionEvent e)
	{
	}

	/**
	 * Aligns the first <code>rows</code> * <code>cols</code>
	 * components of <code>parent</code> in
	 * a grid. Each component in a column is as wide as the maximum
	 * preferred width of the components in that column;
	 * height is similarly determined for each row.
	 * The parent is made just big enough to fit them all.
	 *
	 * @param rows number of rows
	 * @param cols number of columns
	 * @param initialX x location to start the grid at
	 * @param initialY y location to start the grid at
	 * @param xPad x padding between cells
	 * @param yPad y padding between cells
	 */
	private static void makeCompactGrid(Container parent,
		int rows, int cols, int initialX, int initialY, int xPad, int yPad)
	{
		SpringLayout layout;
		try
		{
			layout = (SpringLayout)parent.getLayout();
		}
		catch (ClassCastException exc)
		{
			System.err.println("The first argument to makeCompactGrid must use SpringLayout.");
				return;
		}

		// Align all cells in each column and make them the same width.
		Spring x = Spring.constant(initialX);
		for (int c = 0; c < cols; c++)
		{
			Spring width = Spring.constant(0);
			for (int r = 0; r < rows; r++)
			{
				width = Spring.max(width,
					getConstraintsForCell(r, c, parent, cols).getWidth());
			}
			for (int r = 0; r < rows; r++)
			{
				SpringLayout.Constraints constraints =
					getConstraintsForCell(r, c, parent, cols);
				constraints.setX(x);
				constraints.setWidth(width);
			}
			x = Spring.sum(x, Spring.sum(width, Spring.constant(xPad)));
		}

		// Align all cells in each row and make them the same height.
		Spring y = Spring.constant(initialY);
		for (int r = 0; r < rows; r++)
		{
			Spring height = Spring.constant(0);
			for (int c = 0; c < cols; c++)
			{
				height = Spring.max(height,
					getConstraintsForCell(r, c, parent, cols).getHeight());
			}
			for (int c = 0; c < cols; c++)
			{
				SpringLayout.Constraints constraints =
					getConstraintsForCell(r, c, parent, cols);
				constraints.setY(y);
				constraints.setHeight(height);
			}
			y = Spring.sum(y, Spring.sum(height, Spring.constant(yPad)));
		}

		//Set the parent's size
		SpringLayout.Constraints pCons = layout.getConstraints(parent);
		pCons.setConstraint(SpringLayout.SOUTH, y);
		pCons.setConstraint(SpringLayout.EAST, x);
	}

	private static SpringLayout.Constraints getConstraintsForCell(
		int row, int col, Container parent, int cols)
	{
		SpringLayout layout = (SpringLayout)parent.getLayout();
		Component c = parent.getComponent(row * cols + col);
		return layout.getConstraints(c);
	}

	private void updateShapeBrush()
	{
		Brush brush = null;
		if (!checkBox6.isSelected())
		{
			brush = new SolidBrush(colorPicker6.getColor());
		}
		else
		{
			brush = new GradientBrush(
				colorPicker6.getColor(), colorPicker8.getColor(), 90);
		}

		diagram.getShapeNodeStyle().setBrush(brush);
	}

	private void applyPreset(Preset preset)
	{
		checkBox1.setSelected(preset.UseGlassEffect);
		comboBox1.setSelectedIndex(preset.GlassEffectType);
		checkBox2.setSelected(preset.UsePenAsGlow);
		colorPicker1.setColor(preset.GlowColor);
		colorPicker2.setColor(preset.ReflectionColor);

		checkBox3.setSelected(preset.UseAeroEffect);
		numeric1.setText(Integer.toString(preset.Opacity));
		colorPicker3.setColor(preset.ShadeColor);
		colorPicker4.setColor(preset.InnerOutlineColor);

		colorPicker5.setColor(preset.DiagramBackground);
		colorPicker7.setColor(preset.ShapeBorder);
		colorPicker6.setColor(preset.ShapeBackground1);
		checkBox6.setSelected(preset.UseShapeBackground2);
		colorPicker8.setColor(preset.ShapeBackground2);
	}


	private static final long serialVersionUID = 1L;

	private JPanel panel;
	private JList<Preset> presetsList;
	private JPanel panel2;
	private JCheckBox checkBox1;
	private JPanel glassEffectPanel;
	private JLabel label1;
	private JComboBox<String> comboBox1;
	private JCheckBox checkBox2;
	private JLabel label2;
	private ColorPicker colorPicker1;
	private JLabel label3;
	private ColorPicker colorPicker2;
	private JPanel panel3;
	private JCheckBox checkBox3;
	private JPanel aeroEffectPanel;
	private JLabel label7;
	private JFormattedTextField numeric1;
	private JLabel label6;
	private ColorPicker colorPicker4;
	private JLabel label5;
	private ColorPicker colorPicker3;
	private JPanel bottomPanel;
	private JPanel panel6;
	private JLabel label4;
	private ColorPicker colorPicker5;
	private JLabel label9;
	private ColorPicker colorPicker7;
	private JLabel label8;
	private JPanel panel7;
	private ColorPicker colorPicker6;
	private JCheckBox checkBox6;
	private ColorPicker colorPicker8;
	private DiagramView diagramView;
	private Diagram diagram;
	private JScrollPane diagramPane;

	private GlassEffect glassEffect;
	private AeroEffect aeroEffect;
	private Preset defaultPreset;
}

final class Colors
{
	public static final Color White = new Color(255, 255, 255);
	public static final Color Black = new Color(0, 0, 0);
	public static final Color DarkSlateGray = new Color(0x2F, 0x4F, 0x4F);
	public static final Color LightSteelBlue = new Color(0xB0, 0xC4, 0xDE);
	public static final Color Orange = new Color(0xFF, 0xA5, 0x00);
	public static final Color Red = new Color(0xFF, 0x00, 0x00);
	public static final Color LightCyan = new Color(0xE0, 0xFF, 0xFF);
	public static final Color Gray = new Color(0x80, 0x80, 0x80);
	public static final Color PaleGoldenrod = new Color(0xEE, 0xE8, 0xAA);
}
