import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import com.mindfusion.diagramming.DiagramNode;
import com.mindfusion.diagramming.HandlesStyle;
import com.mindfusion.diagramming.Pen;
import com.mindfusion.diagramming.RenderOptions;
import com.mindfusion.diagramming.SolidBrush;
import com.mindfusion.diagramming.Utilities;


public class CauseNode extends DiagramNode
{
 	public CauseNode(DiagramNode backbone)
	{
		super(backbone.getParent());
		this.backbone = backbone;
		
		setPen(new Pen(1, Color.red));
		setBrush(new SolidBrush(Color.red));
		setHandlesStyle(HandlesStyle.Invisible);
	}
	
	@Override
	protected void draw(Graphics2D g, RenderOptions options)
	{
		int code = (int)'a' + (getZIndex() == 0 ? getParent().getItems().size() : getZIndex()) - 1;
		char c = (char)(code);
		Point2D handle;
		
		Paint paint = g.getPaint();
		Font buffFont = g.getFont();
		g.setPaint(Color.blue);
		g.setFont(new Font("Trebuchet MS", Font.BOLD, 5));

		if (getBounds().getY() < backbone.getBounds().getY())
		{
			g.drawString(String.valueOf(c), (float)(getP1().getX() - 4.5), (float)(getP1().getY() - 1));
			handle = (Point2D)getP2().clone();
		}
		else
		{
			g.drawString(String.valueOf(c), (float)(getP2().getX() - 4.5), (float)(getP2().getY() + 5));
			handle = (Point2D)getP1().clone();
		}
		
		// draw the cause line
		g.setPaint(getPen().getColor());
		g.setStroke(new BasicStroke(getPen().getWidth(), BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
		g.draw(new Line2D.Float((float)getP1().x, (float)getP1().y, (float)getP2().x, (float)getP2().y));

		// draw the movement handle
		g.fill(new Ellipse2D.Float((float)(handle.getX() - 1.5), (float)(handle.getY() - 1.5), 3, 3));
		
		g.setFont(buffFont);
		g.setPaint(paint);
	}

	protected void startCreate(Point2D org)
	{
		super.startCreate(org);
		start = org;
		start.setLocation(start.getX(), 
				backbone.getBounds().getY() + backbone.getBounds().getHeight() / 2);
	}
	
	@Override
	public boolean containsPoint(Point2D point, float threshold)
	{
		return super.containsPoint(point, threshold) &&
			Utilities.distToLineSegment(point, getP1(), getP2()) <= 2 + threshold;
	}

	@Override
	public boolean containsPoint(Point2D point)
	{
		return super.containsPoint(point) &&
			Utilities.distToLineSegment(point, getP1(), getP2()) <= 2;
	}
	
	protected void updateCreate(Point2D current)
	{
		super.updateCreate(current);

		double dy = start.getY() - current.getY();

		if (dy < 0)
		{
			// below the backbone
			setBounds(MainFrame.fromLTRB(start.getX() + dy, start.getY() + 0.5, 
					start.getX(), start.getY() - dy));
		}
		else
		{
			// above the backbone
			setBounds(MainFrame.fromLTRB(start.getX() - dy, start.getY(), 
					start.getX(), start.getY() - dy));
		}
	}
	
	public Point2D.Double getP1()
	{
		Point2D.Double p1;
	
		if (getBounds().getY() < backbone.getBounds().getY())
		{
			p1 = new Point2D.Double(getBounds().getX(), getBounds().getY());
		}
		else
		{
			p1 = new Point2D.Double(getBounds().getMaxX(), getBounds().getY());
		}
	
		return p1;
	}
	
	public Point2D.Double getP2()
	{
		Point2D.Double p2;

		if (getBounds().getY() < backbone.getBounds().getY())
		{
			p2 = new Point2D.Double(getBounds().getMaxX(), getBounds().getMaxY());
		}
		else
		{
			p2 = new Point2D.Double(getBounds().getX(), getBounds().getMaxY());
		}

		return p2;
	}
		
	@Override
	protected void drawShadow(Graphics2D g, RenderOptions options)
	{
	}
	
	@Override
	protected Rectangle2D getRepaintRect(boolean includeConnected)
	{
		Rectangle2D rect = super.getRepaintRect(includeConnected);
		rect.setRect(rect.getX(), rect.getY() - 5, rect.getWidth(),	rect.getHeight() + 10);
		return rect;
	}

	public Rectangle2D getEditRect()
	{
		return null;
	}

	public String getTextToEdit()
	{
		return null;
	}

	public void setEditedText(String newText)
	{
	}
	
	public Rectangle2D getEditRect(Point2D mousePosition)
	{
		return null;
	}
	
	DiagramNode backbone;
	Point2D start;	
}
