
package com.mindfusion.diagramming.samples.demo;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;


public class AddPersonDialog extends JDialog
{
	/**
	 * Initializes a new instance of the AddPersonDialog class with
	 * the specified parent. The dialog is centered within
	 * the parent.
	 */
	public AddPersonDialog(JFrame parent)
	{
		super(parent, true);

		_accepted = false;

		setTitle("Add Person");
		setSize(327, 164);
		setResizable(false);

		// Center in parent
		Rectangle parentBounds = new Rectangle();
		parent.getBounds(parentBounds);

		setLocation((parentBounds.width - getWidth()) / 2,
			(parentBounds.height - getHeight()) / 2);

		_image = null;

		// Initialize frame
		initializeComponent();
	}

	/**
	 * Initializes child controls.
	 */
	private void initializeComponent()
	{
		setLayout(null);

		//
		// _lblImage
		//
		_lblImage = new JLabel();
		_lblImage.setLocation(7, 7);
		_lblImage.setSize(73, 90);
		_lblImage.setBorder(new LineBorder(Color.BLACK, 1));
		_lblImage.addMouseListener
		(
			new MouseAdapter()
			{
				public void mouseClicked(MouseEvent e)
				{
					AddPersonDialog.this.onImageClicked(e);
				}
			}
		);

		//
		// _lblName
		//
		_lblName = new JLabel();
		_lblName.setText("Name:");
		_lblName.setLocation(87, 14);
		_lblName.setSize(46, 14);

		//
		// _txtName
		//
		_txtName = new JTextField();
		_txtName.setLocation(153, 10);
		_txtName.setSize(160, 20);

		//
		// _lblBirthDeath
		//
		_lblBirthDeath = new JLabel();
		_lblBirthDeath.setText("Birth/death:");
		_lblBirthDeath.setLocation(87, 42);
		_lblBirthDeath.setSize(66, 13);

		//
		// _txtBirthDeath
		//
		_txtBirthDeath = new JTextField();
		_txtBirthDeath.setLocation(153, 38);
		_txtBirthDeath.setSize(160, 20);

		//
		// _lblGender
		//
		_lblGender = new JLabel();
		_lblGender.setText("Gender");
		_lblGender.setLocation(87, 70);
		_lblGender.setSize(46, 14);

		//
		// _cbGender
		//
		_cbGender = new JComboBox();
		_cbGender.setModel(new DefaultComboBoxModel(
			new Object[]
			{
				"Unspecified",
				"Male",
				"Female"
			}));
		_cbGender.setLocation(153, 66);
		_cbGender.setSize(121, 21);

		//
		// _btnAdd
		//
		_btnAdd = new JButton();
		_btnAdd.setText("Add");
		_btnAdd.setLocation(240, 104);
		_btnAdd.setSize(72, 20);
		_btnAdd.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					AddPersonDialog.this.onBtnAdd(e);
				}
			}
		);

		//
		// _fileDialog
		//
		_fileDialog = new JFileChooser();

		Container contents = getContentPane();

		contents.add(_lblImage);
		contents.add(_lblName);
		contents.add(_txtName);
		contents.add(_lblBirthDeath);
		contents.add(_txtBirthDeath);
		contents.add(_lblGender);
		contents.add(_cbGender);
		contents.add(_btnAdd);
	}

	/**
	 * Fired when the image has been clicked.
	 */
	private void onImageClicked(MouseEvent e)
	{
		if (_fileDialog.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
		{
			try
			{
				_image = javax.imageio.ImageIO.read(
					_fileDialog.getSelectedFile());
				_lblImage.setIcon(new ImageIcon(_image));
			}
			catch (java.io.IOException ex)
			{
			}
		}
	}

	/**
	 * Handles the Add button click.
	 */
	private void onBtnAdd(ActionEvent e)
	{
		_accepted = true;

		hide();
	}

	/**
	 * Gets whether the user has confirmed the dialog.
	 */
	public boolean getAccepted() { return _accepted; }

	/**
	 * Gets the name of the person.
	 */
	public String getPersonName() { return _txtName.getText(); }

	/**
	 * Gets the birth and death dates of the person.
	 */
	public String getPersonDates() { return _txtBirthDeath.getText(); }

	/**
	 * Gets the image of the person.
	 */
	public Image getPersonImage() { return _image; }

	/**
	 * Gets the gender of the person.
	 */
	public int getGender() { return _cbGender.getSelectedIndex(); }


	private boolean _accepted;

	private JLabel _lblImage;
	private JLabel _lblName;
	private JTextField _txtName;
	private JLabel _lblBirthDeath;
	private JTextField _txtBirthDeath;
	private JLabel _lblGender;
	private JComboBox _cbGender;
	private JButton _btnAdd;

	private JFileChooser _fileDialog;

	private Image _image;
}
