

package com.mindfusion.diagramming.samples.demo;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.text.html.*;

import com.mindfusion.diagramming.*;


public class TourDialog extends JDialog
{
	/**
	 * Initializes a new instance of the TourDialog class with
	 * the specified parent. The dialog is centered within
	 * the parent.
	 */
	public TourDialog(JFrame parent, Diagram flowChart,
		ShapeNode initial)
	{
		super(parent, true);

		setTitle("Conversation");
		setSize(374, 300);
		setResizable(false);

		// Center in parent
		Rectangle parentBounds = new Rectangle();
		parent.getBounds(parentBounds);

		setLocation((parentBounds.width - getWidth()) / 2,
			(parentBounds.height - getHeight()) / 2);

		// Initialize frame
		initializeComponent();

		_flowChart = flowChart;

		setBox(initial);
	}

	/**
	 * Initializes child controls.
	 */
	private void initializeComponent()
	{
		setLayout(null);

		//
		// _text
		//
		_text = new JEditorPane();
		_text.setLocation(7, 7);
		_text.setSize(353, 143);
		_text.setEditable(false);
		_text.setEditorKit(new HTMLEditorKit());

		//
		// _textPane
		//
		_textPane = new JScrollPane(_text);
		_textPane.setBorder(new EtchedBorder(SystemColor.controlHighlight, SystemColor.controlShadow));
		_textPane.setLocation(7, 7);
		_textPane.setSize(353, 143);

		//
		// _optionsList
		//
		_optionsList = new JList(new DefaultListModel());
		_optionsList.setLocation(7, 150);
		_optionsList.setSize(353, 62);
		_optionsList.setBorder(new LineBorder(Color.BLACK, 1));
		_optionsList.setBackground(new Color(0xff, 0xff, 0xee));
		_optionsList.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
		_optionsList.addListSelectionListener
		(
			new ListSelectionListener()
			{
				public void valueChanged(ListSelectionEvent e)
				{
					TourDialog.this.onItemSelected(e);
				}
			}
		);

		//
		// _btnEnd
		//
		_btnEnd = new JButton();
		_btnEnd.setText("End");
		_btnEnd.setLocation(153, 240);
		_btnEnd.setSize(63, 20);
		_btnEnd.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					TourDialog.this.onBtnEnd(e);
				}
			}
		);

		Container contents = getContentPane();

		contents.add(_textPane);
		contents.add(_optionsList);
		contents.add(_btnEnd);
	}

	private void setBox(ShapeNode box)
	{
		_box = box;
		_text.setText(_box.getText());

		// Initialize options
		DefaultListModel model = (DefaultListModel)_optionsList.getModel();
		model.clear();
		for (DiagramLink arrow : _box.getOutgoingLinks())
			model.addElement(arrow.getText());
	}

	/**
	 * Handles the End button click.
	 */
	private void onBtnEnd(ActionEvent e)
	{
		hide();
	}

	private void onItemSelected(ListSelectionEvent e)
	{
		if (!e.getValueIsAdjusting())
		{
			if (_optionsList.getSelectedIndex() != -1)
			{
				setBox((ShapeNode)_box.getOutgoingLinks().get(
					_optionsList.getSelectedIndex()).getDestination());
			}
		}
	}


	private JEditorPane _text;
	private JScrollPane _textPane;
	private JButton _btnEnd;
	private JList _optionsList;

	private Diagram _flowChart;
	private ShapeNode _box;
}
